from os.path import dirname, join

from django.contrib.auth.models import Permission
from test_plus.test import TestCase
from touchtechnology.common.models import SitemapNode
from touchtechnology.common.tests.factories import UserFactory
from touchtechnology.content.models import Placeholder
from touchtechnology.content.tests import factories


class AdminTests(TestCase):
    def setUp(self):
        self.superuser = UserFactory.create(is_staff=True, is_superuser=True)
        self.staff = UserFactory.create(is_staff=True)
        self.staff.user_permissions.set(
            Permission.objects.filter(codename__endswith="_sitemapnode")
        )

        placeholder = Placeholder.objects.get(namespace="context")
        self.root_node = SitemapNode.objects.create(
            title="Simple Test",
            slug="home",
            object=placeholder,
            kwargs={"key": "value"},
        )

    def get_text(self, filename):
        with open(join(dirname(__file__), filename)) as fp:
            text = fp.read()
        return text

    def test_sitemap_index_superuser(self):
        self.assertLoginRequired("admin:content:index")
        with self.login(self.superuser):
            self.get("admin:content:index")
            html = """
                <a role="button" href="/admin/content/application/{}/">
                    <i class="fa fa-pencil fa-fw"></i>
                    Edit
                </a>
            """.format(
                self.root_node.pk
            )
            self.assertResponseContains(html)

    def test_sitemap_index_staff_user(self):
        self.assertLoginRequired("admin:content:index")
        with self.login(self.staff):
            self.get("admin:content:index")
            html = "<span>Simple Test</span>"
            self.assertResponseContains(html)

    def test_edit_page(self):
        with self.login(self.superuser):
            self.assertGoodView("admin:content:page:add")

            copy = """<p>Sample page data.</p>"""
            data = {
                "parent-form-parent": "",
                "parent-form-title": "Sample",
                "parent-form-short_title": "",
                "parent-form-enabled": "1",
                "parent-form-hidden_from_navigation": "0",
                "parent-form-hidden_from_sitemap": "0",
                "parent-form-slug": "",
                "parent-form-slug_locked": "0",
                "form-template": "",
                "form-keywords": "",
                "form-description": "",
                "formset-TOTAL_FORMS": "1",
                "formset-INITIAL_FORMS": "0",
                "formset-MIN_NUM_FORMS": "0",
                "formset-MAX_NUM_FORMS": "1000",
                "formset-0-copy": copy,
                "formset-0-label": "copy",
                "formset-0-sequence": "1",
                "formset-0-id": "",
                "formset-0-page": "",
            }
            self.post("admin:content:page:add", data=data)
            self.response_302()

            node = SitemapNode.objects.latest("pk")
            self.assertEqual(node.get_absolute_url(), "/sample/")

            self.assertEqual(node.object.content.latest("pk").copy, copy)

            # Unfortunately the SitemapNodeMiddleware does not appear to be
            # having an effect?
            # self.get(node.get_absolute_url())
            # self.response_200()


class GoodViewTests(TestCase):
    def setUp(self):
        self.staff = UserFactory.create(is_staff=True, is_superuser=True)
        self.regular = UserFactory.create()

    def test_redirect_list(self):
        factories.RedirectFactory.create()

        with self.login(self.staff):
            self.assertGoodView("admin:content:redirect:list")
            self.response_200()



import base64
import copy
import os.path

from django.test.utils import override_settings
from test_plus import TestCase
from touchtechnology.common.tests.factories import UserFactory
from touchtechnology.news.models import Article

try:
    from io import BytesIO
except ImportError:
    from StringIO import StringIO as BytesIO


TEST_DIR = os.path.join(os.path.dirname(__file__), "html")


class ImageFieldTestCase(TestCase):
    def setUp(self):
        self.superuser = UserFactory.create(is_staff=True, is_superuser=True)

        # Single transparent pixel PNG image, base64 encoded.
        png_data = base64.b64decode(
            "iVBORw0KGgoAAAANSUhEUgAAAAEAAAABAQAAAAA3bvkkAAAACklEQVQImWNoAAAA"
            "ggCByxOyYQAAAABJRU5ErkJggg=="
        )

        self.data_no_image = {
            "headline": "Headline",
            "image": "",
            "abstract": "This is the abstract",
            "copy": "<p>This is the copy</p>",
            "published": "2017-01-01 05:30:00",
        }
        self.data_with_image = copy.copy(self.data_no_image)
        self.data_with_image["image"] = BytesIO(png_data)
        self.data_with_image["image"].name = "pixel.png"

    def test_image_required_no_image(self):
        with self.login(self.superuser):
            self.post("admin:news:article:add", data=self.data_no_image)
        self.assertFormError(
            self.last_response, "form", "image", "This field is required."
        )
        self.assertEqual(0, Article.objects.count())

    def test_image_required_with_image(self):
        with self.login(self.superuser):
            self.post("admin:news:article:add", data=self.data_with_image)
        # print(self.last_response.content.decode("utf-8"))
        self.response_302()
        self.assertEqual(1, Article.objects.count())

    @override_settings(TOUCHTECHNOLOGY_NEWS_IMAGE_REQUIRED=False)
    def test_image_not_required_no_image(self):
        with self.login(self.superuser):
            self.post("admin:news:article:add", data=self.data_no_image)
        # print(self.last_response.content.decode("utf-8"))
        self.response_302()
        self.assertEqual(1, Article.objects.count())

    @override_settings(TOUCHTECHNOLOGY_NEWS_IMAGE_REQUIRED=False)
    def test_image_not_required_with_image(self):
        with self.login(self.superuser):
            self.post("admin:news:article:add", data=self.data_with_image)
        # print(self.last_response.content.decode("utf-8"))
        self.response_302()
        self.assertEqual(1, Article.objects.count())



from django import forms
from django.conf import settings
from django.utils.translation import gettext_lazy as _
from modelforms.forms import ModelForm

from touchtechnology.common.forms.mixins import (
    BootstrapFormControlMixin,
    SuperUserSlugMixin,
)
from touchtechnology.news.models import Article, Category, Translation


class ArticleForm(SuperUserSlugMixin, ModelForm):
    def __init__(self, *args, **kwargs):
        super(ArticleForm, self).__init__(*args, **kwargs)
        if not self.fields["categories"].queryset.count():
            self.fields.pop("categories", None)
        self.fields["image"].required = getattr(
            settings, "TOUCHTECHNOLOGY_NEWS_IMAGE_REQUIRED", True
        )

    class Meta:
        model = Article
        fields = (
            "headline",
            "image",
            "abstract",
            "copy",
            "published",
            "slug",
            "slug_locked",
            "byline",
            "keywords",
            "categories",
            "is_active",
        )


class CategoryForm(SuperUserSlugMixin, ModelForm):
    class Meta:
        model = Category
        fields = (
            "title",
            "short_title",
            "slug",
            "slug_locked",
            "is_active",
            "hidden_from_navigation",
        )


class TranslationForm(BootstrapFormControlMixin, ModelForm):
    class Meta:
        model = Translation
        fields = (
            "locale",
            "headline",
            "abstract",
            "copy",
        )

    locale = forms.ChoiceField(choices=settings.LANGUAGES, label=_("Language"))



from urllib.parse import urljoin

from django.shortcuts import get_object_or_404
from django.urls import include, path
from django.utils.translation import gettext_lazy as _

from touchtechnology.admin.base import AdminComponent
from touchtechnology.common.decorators import staff_login_required_m
from touchtechnology.news.forms import (
    ArticleForm,
    CategoryForm,
    TranslationForm,
)
from touchtechnology.news.models import Article, Category, Translation


class NewsAdminComponent(AdminComponent):
    verbose_name = _("News")
    unprotected = False

    def __init__(self, app, name="news", app_name="news"):
        super(NewsAdminComponent, self).__init__(app, name, app_name)

    def get_urls(self):
        translation_patterns = (
            [
                path("add/", self.edit_translation, name="add"),
                path("<slug:locale>/", self.edit_translation, name="edit"),
                path("<slug:locale>/delete/", self.delete_translation, name="delete"),
                path("<slug:locale>/permission/", self.perms_translation, name="perms"),
            ],
            self.app_name,
        )

        article_patterns = (
            [
                path("", self.list_articles, name="list"),
                path("add/", self.edit_article, name="add"),
                path("<int:pk>/", self.edit_article, name="edit"),
                path("<int:pk>/delete/", self.delete_article, name="delete"),
                path("<int:pk>/permission/", self.perms_article, name="perms"),
                path(
                    "<int:pk>/",
                    include(translation_patterns, namespace="translation"),
                ),
            ],
            self.app_name,
        )

        category_patterns = (
            [
                path("", self.list_categories, name="list"),
                path("add/", self.edit_category, name="add"),
                path("<int:pk>/", self.edit_category, name="edit"),
                path("<int:pk>/delete/", self.delete_category, name="delete"),
                path("<int:pk>/permission/", self.perms_category, name="perms"),
            ],
            self.app_name,
        )

        urlpatterns = [
            path("", self.index, name="index"),
            path("article/", include(article_patterns, namespace="article")),
            path("category/", include(category_patterns, namespace="category")),
        ]
        return urlpatterns

    def dropdowns(self):
        dl = (
            (_("Articles"), self.reverse("article:list"), "newspaper-o"),
            (_("Categories"), self.reverse("category:list"), "tag"),
        )
        return dl

    @staff_login_required_m
    def index(self, request, **kwargs):
        return self.redirect(self.reverse("article:list"))

    # Article views

    @staff_login_required_m
    def list_articles(self, request, **kwargs):
        return self.generic_list(
            request,
            Article,
            paginate_by=25,
            permission_required=True,
            extra_context=kwargs,
        )

    @staff_login_required_m
    def edit_article(self, request, pk=None, **kwargs):
        return self.generic_edit(
            request,
            Article,
            pk=pk,
            form_class=ArticleForm,
            form_kwargs={"user": request.user},
            # permission_required=True,
            post_save_redirect=self.redirect(urljoin(request.path, "..")),
            extra_context=kwargs,
        )

    @staff_login_required_m
    def delete_article(self, request, pk, **kwargs):
        return self.generic_delete(request, Article, pk=pk, permission_required=True)

    @staff_login_required_m
    def perms_article(self, request, pk, **extra_context):
        return self.generic_permissions(request, Article, pk=pk, **extra_context)

    # Translations

    def edit_translation(self, request, pk, locale=None, **extra_context):
        article = get_object_or_404(Article, pk=pk)

        if locale is None:
            instance = Translation(article=article)
        else:
            instance = get_object_or_404(article.translations, locale=locale)

        return self.generic_edit(
            request,
            article.translations,
            instance=instance,
            form_class=TranslationForm,
            post_save_redirect=self.redirect(article.urls["edit"]),
            extra_context=extra_context,
        )

    def delete_translation(self, request, pk, locale, **kwargs):
        instance = get_object_or_404(Translation, article__pk=pk, locale=locale)
        return self.generic_permissions(request, Translation, instance=instance)

    def perms_translation(self, request, pk, locale, **extra_context):
        instance = get_object_or_404(Translation, article__pk=pk, locale=locale)
        return self.generic_permissions(
            request,
            Translation,
            instance=instance,
            post_save_redirect=self.redirect(instance.article.urls["edit"]),
            **extra_context
        )

    # Category views

    @staff_login_required_m
    def list_categories(self, request, **kwargs):
        return self.generic_list(
            request,
            Category,
            paginate_by=25,
            permission_required=True,
            extra_context=kwargs,
        )

    @staff_login_required_m
    def edit_category(self, request, pk=None, **kwargs):
        return self.generic_edit(
            request,
            Category,
            pk=pk,
            form_class=CategoryForm,
            form_kwargs={"user": request.user},
            permission_required=True,
            post_save_redirect=self.redirect(urljoin(request.path, "..")),
            extra_context=kwargs,
        )

    @staff_login_required_m
    def delete_category(self, request, pk, **kwargs):
        return self.generic_delete(request, Category, pk=pk, permission_required=True)

    @staff_login_required_m
    def perms_category(self, request, pk, **extra_context):
        return self.generic_permissions(
            request,
            Category,
            pk=pk,
            post_save_redirect=self.redirect(
                self.reverse("category:edit", kwargs={"pk": pk})
            ),
            **extra_context
        )



# -*- coding: utf-8 -*-
# Generated by Django 1.11.17 on 2019-01-04 13:43
from __future__ import unicode_literals

from django.db import migrations
from tournamentcontrol.competition.signals.matches import match_saved_handler


def resave_all_matches(apps, schema_editor):
    Match = apps.get_model("competition", "Match")
    for m in Match.objects.all():
        # We have to manually exercise the match_saved_handler code because migrations use a proxy
        # model (ie. the "frozen" representation) and our handlers are attached on the fair-dinkum
        # model.  This will work because we don't use the `sender` in our handler - otherwise we'd
        # need to refactor the handler.
        match_saved_handler(None, m, False)


def noop(apps, schema_editor):
    pass


class Migration(migrations.Migration):

    dependencies = [("competition", "0031_model_definition_sync")]

    operations = [migrations.RunPython(resave_all_matches, noop)]



import logging

logger = logging.getLogger(__name__)

NAME = 'Competitions'
INSTALL = (
    'CompetitionSite',
    'MultiCompetitionSite',
    'RegistrationSite',
    'TournamentCalculatorSite',
    'RankingSite',
)

__version__ = '2.4.14'

logger.debug('"%s"/"%s"' % (NAME, __version__))

default_app_config = 'tournamentcontrol.competition.apps.CompetitionConfig'
